==> tin-1.00.tar.gz: tin-1.00/hashstr.c <==
/*
 *  Project   : tin - a visual threaded usenet newsreader
 *  Module    : hashstr.c
 *  Author    : R.Skrenta
 *  Created   : 01-04-91
 *  Updated   : 10-08-91
 *  Release   : 1.0
 *  Notes     :
 *  Copyright : (c) Copyright 1991 by Rich Skrenta
 *	            You may  freely  copy or  redistribute  this software,
 *              so  long as there is no profit made from its use, sale
 *              trade or  reproduction.  You may not change this copy-
 *              right notice, and it must be included in any copy made
 */

#include	<stdio.h>
#ifdef BSD
#include	<strings.h>
#else
#include	<string.h>
#include	<malloc.h>
#endif

/*
 *  Maintain a table of all strings we have seen.
 *  If a new string comes in, add it to the table and return a pointer
 *  to it.  If we've seen it before, just return the pointer to it.
 *
 *  Usage:  hash_str("some string") returns char *
 *
 *  Spillovers are chained on the end
 *
 *  !!! NOTE: CHECK OUT add_string FOR THE *iptr TRICK THAT IS LATER !!!
 *  !!! USED IN dump_index (art.c)                                   !!!
 *
 */


/*
 *  Arbitrary table size, but make sure it's prime!
 */

/* #define		TABLE_SIZE	1409	*/

#define		TABLE_SIZE	2411



struct hashnode {
	char *s;			/* the string we're saving */
	struct hashnode *next;		/* chain for spillover */
};

struct hashnode *table[TABLE_SIZE];

extern char *my_malloc();
struct hashnode *add_string();


char *hash_str(s)
char *s;
{
	struct hashnode *p;	/* used to descend the spillover structs */
	long h;			/* result of hash:  index into hash table */

	if (s == NULL)
		return(NULL);

	{
		char *t = s;

		h = *t++;
		while (*t)
			h = ((h << 1) ^ *t++) % TABLE_SIZE;
	/*		h = (h * 128 + *t++) % TABLE_SIZE;	*/
	}

	p = table[h];

	if (p == NULL) {
		table[h] = add_string(s);
		return table[h]->s;
	}

	while (1) {
		if (strcmp(s, p->s) == 0)
			return(p->s);

		if (p->next == NULL) {
			p->next = add_string(s);
			return p->next->s;
		} else
			p = p->next;
	}
}


struct hashnode *add_string(s)
char *s;
{
	struct hashnode *p;
	extern char *strcpy();
	int *iptr;

	p = (struct hashnode *) my_malloc(sizeof(*p));

	p->next = NULL;
	iptr = (int *) my_malloc((unsigned) strlen(s) + sizeof(int) + 1);
	*iptr++ = -1;
	p->s = (char *) iptr;
	strcpy(p->s, s);
	return(p);
}


void hash_init ()
{
	int i;

	for (i = 0; i < TABLE_SIZE; i++)
		table[i] = NULL;
}


void hash_reclaim ()
{
	int i;
	struct hashnode *p, *next;
	int *iptr;

	for (i = 0; i < TABLE_SIZE; i++)
		if (table[i] != NULL) {
			p = table[i];
			while (p != NULL) {
				next = p->next;
				iptr = (int *) p->s;
				free(--iptr);
				free(p);
				p = next;
			}
			table[i] = NULL;
		}
}

==> tin-1.02.tar.gz: tin-1.02/hashstr.c <==
/*
 *  Project   : tin - a visual threaded usenet newsreader
 *  Module    : hashstr.c
 *  Author    : R.Skrenta
 *  Created   : 01-04-91
 *  Updated   : 05-09-91
 *  Release   : 1.0
 *  Notes     :
 *  Copyright : (c) Copyright 1991 by Rich Skrenta
 *	            You may  freely  copy or  redistribute  this software,
 *              so  long as there is no profit made from its use, sale
 *              trade or  reproduction.  You may not change this copy-
 *              right notice, and it must be included in any copy made
 */

#include	<stdio.h>
#ifdef BSD
#include	<strings.h>
#else
#include	<string.h>
#include	<malloc.h>
#endif
#include	"proto.h"

/*
 *  Maintain a table of all strings we have seen.
 *  If a new string comes in, add it to the table and return a pointer
 *  to it.  If we've seen it before, just return the pointer to it.
 *
 *  Usage:  hash_str("some string") returns char *
 *
 *  Spillovers are chained on the end
 *
 *  !!! NOTE: CHECK OUT add_string FOR THE *iptr TRICK THAT IS LATER !!!
 *  !!! USED IN dump_index (art.c)                                   !!!
 *
 */

/*
 *  Arbitrary table size, but make sure it's prime!
 */

/* #define		TABLE_SIZE	1409	*/

#define		TABLE_SIZE	2411


struct hashnode {
	char *s;			/* the string we're saving */
	struct hashnode *next;		/* chain for spillover */
};

struct hashnode *table[TABLE_SIZE];


char *hash_str (s)
	char *s;
{
	struct hashnode *p;	/* used to descend the spillover structs */
	long h;				/* result of hash:  index into hash table */

	if (s == (char *) 0) {
		return ((char *) 0);
	}

	{
		unsigned char *t = (unsigned char *) s;

		h = *t++;
		while (*t)
			h = ((h << 1) ^ *t++) % TABLE_SIZE;
	}

	p = table[h];

	if (p == NULL) {
		table[h] = add_string (s);
		return table[h]->s;
	}

	while (1) {
		if (strcmp (s, p->s) == 0) {
			return (p->s);
		}

		if (p->next == (struct hashnode *) 0) {
			p->next = add_string (s);
			return p->next->s;
		} else
			p = p->next;
	}
	/* NOTREACHED */
}


struct hashnode *add_string (s)
	char *s;
{
	struct hashnode *p;
	int *iptr;

	p = (struct hashnode *) my_malloc (sizeof (*p));

	p->next = (struct hashnode *) 0;
	iptr = (int *) my_malloc ((unsigned) strlen (s) + sizeof (int)+1);
	*iptr++ = -1;
	p->s = (char *) iptr;
	strcpy (p->s, s);
	return (p);
}


void hash_init ()
{
	int i;

	for (i = 0; i < TABLE_SIZE; i++) {
		table[i] = (struct hashnode *) 0;
	}
}


void hash_reclaim ()
{
	int i;
	struct hashnode *p, *next;
	int *iptr;

	for (i = 0; i < TABLE_SIZE; i++)
		if (table[i] != (struct hashnode *) 0) {
			p = table[i];
			while (p != (struct hashnode *) 0) {
				next = p->next;
				if (p->s != (char *) 0) {
					iptr = (int *) p->s;
					free ((char *) --iptr);
				}
				free ((char *) p);
				p = next;
			}
			table[i] = (struct hashnode *) 0;
		}
}
